/*
 * Copyright (c) 2012-present Marvelution B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
define('jira-jenkins-integration/admin/sites', [
	'jquery',
	'aui/flag',
	'jira-jenkins-integration/utils'
], function (jquery, flag, utils) {
	'use strict';

	var DIALOG_ID = '#site-dialog';
	var DELETE_DIALOG_ID = '#site-delete-dialog';
	var DELETE_JOBS_DIALOG_ID = '#site-jobs-delete-dialog';

	var sites = {
		init: function () {
			jquery(document).on('click', '.add-site', function () {
				sites.addSite();
			});
			jquery(document).on('click', '.edit-site', function (event) {
				sites.doDropdownActionForSite(event, sites.editSite);
			});
			jquery(document).on('click', '.delete-site', function (event) {
				sites.doDropdownActionForSite(event, sites.deleteSite);
			});
			jquery(document).on('click', '.delete-site-jobs', function (event) {
				sites.doDropdownActionForSite(event, sites.deleteJobs);
			});
			jquery(document).on('click', '.sync-site', function (event) {
				sites.doDropdownActionForSite(event, sites.syncSite);
			});
			jquery(document).on('click', '.reregister-site', function (event) {
				sites.doDropdownActionForSite(event, sites.reregisterSite);
			});
			window.setTimeout(function () {
				jquery('[jenkins-site-id]').each(function () {
					sites.checkSiteStatus(jquery(this).attr('jenkins-site-id'));
				});
			}, 1000);
		},

		doDropdownActionForSite: function (event, callback) {
			var siteId = jquery(event.target).closest('[jenkins-site-id]').attr('jenkins-site-id');
			if (siteId !== undefined) {
				callback(siteId);
			}
		},

		getSiteContainerSelector: function (siteId) {
			return '[jenkins-site-id="' + siteId + '"]';
		},

		addSite: function () {
			sites.showSiteDialog();
			return false;
		},

		editSite: function (siteId) {
			jquery.ajax({
				type: 'GET',
				dataType: 'json',
				url: AJS.contextPath() + '/rest/jenkins/1.0/site/' + siteId,
				success: function (site) {
					if (site.displayUrl === site.rpcUrl) {
						site.displayUrl = '';
					}
					sites.showSiteDialog(site);
				}
			}).error(function () {
				flag({
					type: 'error',
					title: AJS.I18n.getText('site.load.failed.title'),
					body: AJS.I18n.getText('site.load.failed.message')
				});
			});
		},

		showSiteDialog: function (site) {
			AJS.dialog2(JJI.Templates.Admin.Sites.siteDialog(site)).show();
			jquery('#changeToken').off('click').on('click', function () {
				if (jquery(this).is(':checked')) {
					jquery('#token').parent('.field-group').removeClass('hidden');
				} else {
					jquery('#token').parent('.field-group').addClass('hidden');
				}
			});
			jquery(DIALOG_ID + ' .aui-button-link').off('click').on('click', function () {
				AJS.dialog2(DIALOG_ID).hide();
				return false;
			});
			jquery(DIALOG_ID + ' .aui-button-primary').off('click').on('click', function () {
				utils.blanketDialog(DIALOG_ID);
				jquery(DIALOG_ID + ' div.error').remove();
				var site = utils.serializeObject(jquery(DIALOG_ID + ' form'));
				site['changeToken'] = undefined;
				jquery.ajax({
					type: 'POST',
					dataType: 'json',
					contentType: 'application/json',
					url: AJS.contextPath() + '/rest/jenkins/1.0/site' + (site.id !== undefined ? '/' + site.id : ''),
					data: JSON.stringify(site),
					success: function (site) {
						flag({
							type: 'success',
							close: 'auto',
							title: AJS.I18n.getText('site.save.success.title', site.name)
						});
						AJS.dialog2(DIALOG_ID).hide();
						location.reload();
					}
				}).error(function (xhr) {
					utils.unblanketDialog(DIALOG_ID);
					var data = JSON.parse(xhr.responseText);
					if (data.errors !== undefined) {
						jquery.each(data.errors, function (a, error) {
							jquery('#' + error.field).parent().append(aui.form.fieldError(error));
						});
					} else {
						AJS.messages.error('#site-errors', {
							insert: 'prepend',
							title: AJS.I18n.getText('site.save.failed.title'),
							body: utils.getErrorMessage(data)
						});
					}
				});
				return false;
			});
		},

		getSiteName: function (id) {
			return jquery(sites.getSiteContainerSelector(id) + ' .site-name').text();
		},

		deleteSite: function (id) {
			var name = sites.getSiteName(id);
			AJS.dialog2(JJI.Templates.Admin.Sites.deleteSiteDialog({'name': name})).show();
			jquery(DELETE_DIALOG_ID + ' .aui-button-link').on('click', function () {
				AJS.dialog2(DELETE_DIALOG_ID).hide();
				return false;
			});
			jquery(DELETE_DIALOG_ID + ' .aui-button-primary').on('click', function () {
				utils.blanketDialog(DELETE_DIALOG_ID);
				jquery.ajax({
					type: 'DELETE',
					url: AJS.contextPath() + '/rest/jenkins/1.0/site/' + id,
					success: function () {
						flag({
							type: 'success',
							close: 'auto',
							title: AJS.I18n.getText('site.delete.success.message', name)
						});
						AJS.dialog2(DELETE_DIALOG_ID).hide();
						if (jquery(sites.getSiteContainerSelector(id)).hasClass('aui-page-header')) {
							location.replace(location.href.substring(0, location.href.lastIndexOf(id) - 1));
						} else {
							location.reload();
						}
					}
				}).error(function () {
					utils.unblanketDialog(DELETE_DIALOG_ID);
					flag({
						type: 'error',
						title: AJS.I18n.getText('site.delete.failed.title', name),
						body: AJS.I18n.getText('site.delete.failed.message')
					});
					AJS.dialog2(DELETE_DIALOG_ID).hide();
				});
				return false;
			});
		},

		deleteJobs: function (id) {
			var name = sites.getSiteName(id);
			AJS.dialog2(JJI.Templates.Admin.Sites.deleteSiteJobsDialog({'name': name})).show();
			jquery(DELETE_JOBS_DIALOG_ID + ' .aui-button-link').on('click', function () {
				AJS.dialog2(DELETE_JOBS_DIALOG_ID).hide();
				return false;
			});
			jquery(DELETE_JOBS_DIALOG_ID + ' .aui-button-primary').on('click', function () {
				utils.blanketDialog(DELETE_JOBS_DIALOG_ID);
				jquery.ajax({
					type: 'DELETE',
					url: AJS.contextPath() + '/rest/jenkins/1.0/site/' + id + '/jobs',
					success: function () {
						flag({
							type: 'success',
							close: 'auto',
							title: AJS.I18n.getText('site.jobs.delete.success.message', name)
						});
						AJS.dialog2(DELETE_JOBS_DIALOG_ID).hide();
						location.reload();
					}
				}).error(function () {
					utils.unblanketDialog(DELETE_JOBS_DIALOG_ID);
					flag({
						type: 'error',
						title: AJS.I18n.getText('site.jobs.delete.failed.title', name),
						body: AJS.I18n.getText('site.jobs.delete.failed.message')
					});
					AJS.dialog2(DELETE_JOBS_DIALOG_ID).hide();
				});
				return false;
			});
		},

		syncSite: function (id) {
			const name = sites.getSiteName(id);
			jquery.ajax({
				type: 'POST',
				dataType: 'json',
				contentType: 'application/json',
				url: AJS.contextPath() + '/rest/jenkins/1.0/site/' + id + '/sync',
				success: function () {
					flag({
						type: 'success',
						close: 'auto',
						title: AJS.I18n.getText('sync.site.successful', name)
					});
				}
			}).error(function () {
				flag({
					type: 'error',
					title: AJS.I18n.getText('sync.site.failed', name)
				});
			});
		},

		reregisterSite: function (id) {
			const name = sites.getSiteName(id);
			jquery.ajax({
				type: 'POST',
				dataType: 'json',
				contentType: 'application/json',
				url: AJS.contextPath() + '/rest/jenkins/1.0/site/' + id + '/register',
				success: function () {
					flag({
						type: 'success',
						close: 'auto',
						title: AJS.I18n.getText('reregister.site.successful', name)
					});
				}
			}).error(function () {
				flag({
					type: 'error',
					title: AJS.I18n.getText('reregister.site.failed', name)
				});
			});
		},

		checkSiteStatus: function (siteId) {
			jquery.ajax({
				type: 'GET',
				dataType: 'json',
				url: AJS.contextPath() + '/rest/jenkins/1.0/site/' + siteId + '/status',
				success: function (data) {
					var context = "#jji-message-bar";
					var status = data.status || '';
					var pluginInstalled = data.pluginInstalled || false;
					var site = data.site || {};
					if (status === 'OFFLINE') {
						AJS.messages.warning(context, {
							body: AJS.I18n.getText('site.offline.message', site.name)
						});
					} else if (status === 'NOT_ACCESSIBLE') {
						AJS.messages.error(context, {
							title: AJS.I18n.getText('site.not.accessible.title', site.name),
							body: AJS.I18n.getText('site.not.accessible.message', site.name)
						});
					} else if (!pluginInstalled) {
						AJS.messages.hint(context, {
							title: AJS.I18n.getText('site.plugin.missing.title', site.name),
							body: AJS.I18n.getText('site.plugin.missing.message', AJS.contextPath)
						});
					}
				}
			});
		}
	};

	return sites;
});
